@@ -20,6 +20,7 @@
 #include "cheese-config.h"
 
 #include <glib/gi18n.h>
+#include <clutter-gst/clutter-gst.h>
 
 #include "cheese-widget.h"
 #include "cheese-camera.h"
@@ -81,29 +82,27 @@ cheese_widget_load_pixbuf (GtkWidget  *widget,
 }
 
 static gboolean
-cheese_widget_logo_expose (GtkWidget      *w,
-                           GdkEventExpose *event,
-                           gpointer        user_data)
+cheese_widget_logo_draw (GtkWidget      *w,
+                         cairo_t        *cr,
+                         gpointer        user_data)
 {
   const char   *icon_name;
   GdkPixbuf    *pixbuf, *logo;
   GError       *error = NULL;
-  cairo_t      *cr;
   GtkAllocation allocation;
   guint         s_width, s_height, d_width, d_height;
   float         ratio;
 
   gtk_widget_get_allocation (w, &allocation);
 
-  gdk_draw_rectangle (gtk_widget_get_window (w),
-                      gtk_widget_get_style (w)->black_gc, TRUE,
-                      0, 0, allocation.width, allocation.height);
-  icon_name = g_object_get_data (G_OBJECT (w), "icon-name");
-  if (icon_name == NULL)
-    return FALSE;
-
-  cr = gdk_cairo_create (gtk_widget_get_window (w));
   cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+
+  icon_name = g_object_get_data (G_OBJECT (w), "icon-name");
+  if (icon_name == NULL) {
+    cairo_paint (cr);
+    return FALSE;
+  }
+
   cairo_rectangle (cr, 0, 0, allocation.width, allocation.height);
 
   d_width  = allocation.width;
@@ -136,7 +135,6 @@ cheese_widget_logo_expose (GtkWidget      *w,
 
   gdk_cairo_set_source_pixbuf (cr, logo, (allocation.width - s_width) / 2, (allocation.height - s_height) / 2);
   cairo_paint (cr);
-  cairo_destroy (cr);
 
   g_object_unref (logo);
   g_object_unref (pixbuf);
@@ -147,25 +145,22 @@ cheese_widget_logo_expose (GtkWidget      *w,
 static void
 cheese_widget_spinner_invert (GtkWidget *spinner, GtkWidget *parent)
 {
-  GtkStyle *style;
+  GtkStyleContext *context;
   guint     i;
 
   for (i = GTK_STATE_NORMAL; i <= GTK_STATE_INSENSITIVE; i++)
   {
-    GdkColor *fg, *bg;
+    GdkRGBA fg, bg;
 
-    style = gtk_widget_get_style (spinner);
-    fg    = gdk_color_copy (&style->fg[i]);
-    bg    = gdk_color_copy (&style->bg[i]);
+    context = gtk_widget_get_style_context (spinner);
+    gtk_style_context_get_color (context, gtk_style_context_get_state (context), &fg);
+    gtk_style_context_get_background_color (context, gtk_style_context_get_state (context), &bg);
 
-    gtk_widget_modify_fg (spinner, i, bg);
-    gtk_widget_modify_bg (spinner, i, fg);
+    gtk_widget_override_color (spinner, i, &bg);
+    gtk_widget_override_background_color (spinner, i, &fg);
 
-    gtk_widget_modify_fg (parent, i, bg);
-    gtk_widget_modify_bg (parent, i, fg);
-
-    gdk_color_free (fg);
-    gdk_color_free (bg);
+    gtk_widget_override_color (parent, i, &bg);
+    gtk_widget_override_background_color (parent, i, &fg);
   }
 }
 
@@ -178,8 +173,8 @@ cheese_widget_set_problem_page (CheeseWidget *widget,
   gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), PROBLEM_PAGE);
   g_object_set_data_full (G_OBJECT (priv->problem),
                           "icon-name", g_strdup (icon_name), g_free);
-  g_signal_connect (priv->problem, "expose-event",
-                    G_CALLBACK (cheese_widget_logo_expose), widget);
+  g_signal_connect (priv->problem, "draw",
+                    G_CALLBACK (cheese_widget_logo_draw), widget);
 }
 
 static void
@@ -358,7 +353,6 @@ setup_camera (CheeseWidget *widget)
 static void
 cheese_widget_realize (GtkWidget *widget)
 {
-  GdkWindow           *window;
   CheeseWidgetPrivate *priv = CHEESE_WIDGET_GET_PRIVATE (widget);
 
   GTK_WIDGET_CLASS (cheese_widget_parent_class)->realize (widget);
@@ -366,35 +360,13 @@ cheese_widget_realize (GtkWidget *widget)
   gtk_spinner_start (GTK_SPINNER (priv->spinner));
 
   gtk_widget_realize (priv->screen);
-  window = gtk_widget_get_window (priv->screen);
-  if (!gdk_window_ensure_native (window))
-  {
-    /* abort: no native window, no xoverlay, no cheese. */
-    g_warning ("Could not create a native X11 window for the drawing area");
-    goto error;
-  }
 
-  gdk_window_set_back_pixmap (gtk_widget_get_window (priv->screen), NULL, FALSE);
-  gtk_widget_set_app_paintable (priv->screen, TRUE);
-  gtk_widget_set_double_buffered (priv->screen, FALSE);
-
-  if (!g_thread_create ((GThreadFunc) setup_camera, widget, FALSE, &priv->error))
-  {
-    g_warning ("Failed to create setup thread: %s", priv->error->message);
-    goto error;
-  }
+  setup_camera (CHEESE_WIDGET (widget));
 
   gtk_widget_set_app_paintable (priv->problem, TRUE);
   gtk_widget_realize (priv->problem);
-  gdk_window_set_back_pixmap (gtk_widget_get_window (priv->problem), NULL, FALSE);
 
   return;
-
-error:
-  gtk_spinner_stop (GTK_SPINNER (priv->spinner));
-  priv->state = CHEESE_WIDGET_STATE_ERROR;
-  g_object_notify (G_OBJECT (widget), "state");
-  cheese_widget_set_problem_page (CHEESE_WIDGET (widget), "error");
 }
 
 static void
@@ -403,6 +375,9 @@ cheese_widget_class_init (CheeseWidgetClass *klass)
   GObjectClass   *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
+  gtk_clutter_init (NULL, NULL);
+  clutter_gst_init (NULL, NULL);
+
   object_class->finalize = cheese_widget_finalize;
 #if 0
   object_class->set_property = cheese_widget_set_property;
